#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2014-2017 European Commission (JRC);
# Licensed under the EUPL (the 'Licence');
# You may not use this work except in compliance with the Licence.
# You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
"""
It contains classes and functions of general utility.
These are python-specific utilities and hacks - general data-processing or
numerical operations.
"""


from collections import OrderedDict, defaultdict
from contextlib import contextmanager
import inspect
import io
import re
import statistics
import sys
import yaml

import networkx as nx
import numpy as np


try:
    isidentifier = str.isidentifier
except AttributeError:
    isidentifier = re.compile(r'[a-z_]\w*$', re.I).match

__all__ = [
    'grouper', 'sliding_window', 'median_filter', 'reject_outliers',
    'clear_fluctuations', 'argmax', 'derivative'
]


class Constants(dict):
    @nx.utils.open_file(1, mode='rb')
    def load(self, file, **kw):
        self.from_dict(yaml.load(file, **kw))
        return self

    @nx.utils.open_file(1, mode='w')
    def dump(self, file, default_flow_style=False, **kw):
        d = self.to_dict()
        yaml.dump(d, file, default_flow_style=default_flow_style, **kw)

    def from_dict(self, d):
        for k, v in sorted(d.items()):
            if isinstance(v, Constants):
                o = getattr(self, k, Constants())
                if isinstance(o, Constants):
                    v = o.from_dict(v)
                elif issubclass(o.__class__, Constants):
                    v = o().from_dict(v)
                if not v:
                    continue
            elif hasattr(self, k) and getattr(self, k) == v:
                continue
            setattr(self, k, v)
            self[k] = v

        return self

    def to_dict(self, base=None):
        pr = {} if base is None else base
        s = (set(dir(self)) - set(dir(Constants)))
        for n in s.union(self.__class__.__dict__.keys()):
            if n.startswith('__'):
                continue
            v = getattr(self, n)
            if inspect.ismethod(v) or inspect.isbuiltin(v):
                continue
            try:
                if isinstance(v, Constants):
                    v = v.to_dict(base=Constants())
                elif issubclass(v, Constants):
                    v = v.to_dict(v, base=Constants())
            except TypeError:
                pass
            pr[n] = v
        return pr


def argmax(values, **kws):
    return np.argmax(np.append(values, [True]), **kws)


def grouper(iterable, n):
    """
    Collect data into fixed-length chunks or blocks.

    :param iterable:
        Iterable object.
    :param iterable: iter

    :param n:
        Length chunks or blocks.
    :type n: int
    """
    args = [iter(iterable)] * n
    return zip(*args)


def sliding_window(xy, dx_window):
    """
    Returns a sliding window (of width dx) over data from the iterable.

    :param xy:
        X and Y values.
    :type xy: list[(float, float) | list[float]]

    :param dx_window:
        dX window.
    :type dx_window: float

    :return:
        Data (x & y) inside the time window.
    :rtype: generator
    """

    dx = dx_window / 2
    it = iter(xy)
    v = next(it)
    window = []

    for x, y in xy:
        # window limits
        x_dn = x - dx
        x_up = x + dx

        # remove samples
        window = [w for w in window if w[0] >= x_dn]

        # add samples
        while v and v[0] <= x_up:
            window.append(v)
            try:
                v = next(it)
            except StopIteration:
                v = None

        yield window


def median_filter(x, y, dx_window, filter=statistics.median_high):
    """
    Calculates the moving median-high of y values over a constant dx.

    :param x:
        x data.
    :type x: Iterable

    :param y:
        y data.
    :type y: Iterable

    :param dx_window:
        dx window.
    :type dx_window: float

    :param filter:
        Filter function.
    :type filter: function

    :return:
        Moving median-high of y values over a constant dx.
    :rtype: numpy.array
    """

    xy = list(zip(x, y))
    Y = []
    add = Y.append
    for v in sliding_window(xy, dx_window):
        add(filter(list(zip(*v))[1]))
    return np.array(Y)


def get_inliers(x, n=1, med=np.median, std=np.std):
    x = np.asarray(x)
    if not x.size:
        return np.zeros_like(x, dtype=bool), np.nan, np.nan
    m, s = med(x), std(x)

    y = n > (np.abs(x - m) / s)
    return y, m, s


def reject_outliers(x, n=1, med=np.median, std=np.std):
    """
    Calculates the median and standard deviation of the sample rejecting the
    outliers.

    :param x:
        Input data.
    :type x: Iterable

    :param n:
        Number of standard deviations.
    :type n: int

    :param med:
        Median function.
    :type med: function, optional

    :param std:
        Standard deviation function.
    :type std: function, optional

    :return:
        Median and standard deviation.
    :rtype: (float, float)
    """

    y, m, s = get_inliers(x, n=n, med=med, std=std)

    if y.any():
        y = np.asarray(x)[y]

        m, s = med(y), std(y)

    return m, s


def ret_v(v):
    """
    Returns a function that return the argument v.

    :param v:
        Object to be returned.
    :type v: object

    :return:
        Function that return the argument v.
    :rtype: function
    """

    return lambda: v


def clear_fluctuations(times, gears, dt_window):
    """
    Clears the gear identification fluctuations.

    :param times:
        Time vector.
    :type times: numpy.array

    :param gears:
        Gear vector.
    :type gears: numpy.array

    :param dt_window:
        Time window.
    :type dt_window: float

    :return:
        Gear vector corrected from fluctuations.
    :rtype: numpy.array
    """

    xy = [list(v) for v in zip(times, gears)]

    for samples in sliding_window(xy, dt_window):

        up, dn = False, False

        x, y = zip(*samples)

        for k, d in enumerate(np.diff(y)):
            if d > 0:
                up = True
            elif d < 0:
                dn = True

            if up and dn:
                m = statistics.median_high(y)
                for v in samples:
                    v[1] = m
                break

    return np.array([y[1] for y in xy])


def _err(v, y1, y2, r, l):
    import sklearn.metrics as sk_met

    return sk_met.mean_absolute_error(_ys(y1, v) + _ys(y2, l - v), r)


def _ys(y, n):
    if n:
        return (y,) * int(n)
    return ()


def derivative(x, y, dx=1, order=3, k=1):
    """
    Find the 1-st derivative of a spline at a point.

    Given a function, use a central difference formula with spacing `dx` to
    compute the `n`-th derivative at `x0`.

    :param x:
    :param y:
    :param dx:
    :param order:
    :param k:
    :return:
    """
    import scipy.misc as sci_misc
    import scipy.interpolate as sci_itp

    func = sci_itp.InterpolatedUnivariateSpline(x, y, k=k)

    return sci_misc.derivative(func, x, dx=dx, order=order)


@contextmanager
def stds_redirected(stdout=None, stderr=None):
    captured_out = io.StringIO() if stdout is None else stdout
    captured_err = io.StringIO() if stderr is None else stderr
    orig_out, sys.stdout = sys.stdout, captured_out
    orig_err, sys.stderr = sys.stderr, captured_err

    yield captured_out, captured_err

    sys.stdout, sys.stderr = orig_out, orig_err


_key_value_regex = re.compile(r'^\s*([/_A-Za-z][\w/\.]*)\s*([+*?:@]?)=\s*(.*?)\s*$')



##############################
## Maintain ordered YAML
#  from http://stackoverflow.com/a/21912744
#
_MAPTAG = yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG


def _construct_ordered_dict(loader, node):
    loader.flatten_mapping(node)
    return OrderedDict(loader.construct_pairs(node))


def _ordered_dict_representer(dumper, data):
    return dumper.represent_mapping(
        yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
        data.items())


def yaml_load(stream, Loader=yaml.SafeLoader):
    class OrderedLoader(Loader):
        pass

    OrderedLoader.add_constructor(_MAPTAG, _construct_ordered_dict)
    return yaml.load(stream, OrderedLoader)


def yaml_dump(data, stream=None, Dumper=yaml.SafeDumper, **kwds):
    class OrderedDumper(Dumper):
        pass

    OrderedDumper.add_representer(OrderedDict, _ordered_dict_representer)
    return yaml.dump(data, stream, OrderedDumper, **kwds)


def setup_yaml_ordered():
    """
    Invoke it once it to enable app-wide ordered yaml.

    From http://stackoverflow.com/a/8661021 """
    yaml.add_representer(OrderedDict, _ordered_dict_representer)
    yaml.add_representer(defaultdict, _ordered_dict_representer)
    yaml.add_representer(tuple, yaml.SafeDumper.represent_list)
    yaml.add_constructor(_MAPTAG, _construct_ordered_dict)



from co2mpas_driver.common import vehicle_specs_class as vcc


def load_db_to_dictionary(name):
    """
    input: file (without .csv)
    return: dict of dictionaries, every dict item is a car, every dict key is the car's characteristic's name, and the
    value returned from the key is the value of the characteristic.

    :param name:
    :return:
    """

    ##First replace any , with - in the csv
    file = open(name+'.csv', 'r', encoding="ISO-8859-1")
    A = file.readline()
    Characteristics = []
    Out = {}

    ##create the list of the names of the characteristic
    while A != '':
        k = A.find(',')
        if k == -1:
            Characteristics.append(A)
            A = ''
        else:
            Characteristics.append(A[:k])
            A = A[k + 1:]

    ##for every line of the csv creates a dictionary
    ##then for every cell of the line, stores the value to the dictionary using the characteristic's name as key
    for line in file:
        C = {}
        for i in range(len(Characteristics) - 1):
            k = line.find(',')
            C[Characteristics[i]] = line[:k]
            line = line[k + 1:]
        C[Characteristics[-1].rstrip('\n')] = line.rstrip('\n')
        Out[int(C[Characteristics[0]])] = C

    file.close()
    return Out


def get_vehicle_from_db(db_dict, car_id, **kwargs):
    """

    kwargs can be:
    lco = True          # Light co2mpas is to be used, so the relevant parameters must be imported
    electric = True     # The vehicle is an EV

    :param db_dict: dict of dicts ,A dictionary produced by the "load_db_to_dictionary@" function
    :param car_id: int, The car id in the db
    :return: veh_specs object
    """

    my_car = db_dict[car_id]

    my_car_specs = vcc.veh_specs(my_car, **kwargs)

    return my_car_specs


import logging
import numpy as np
from os import path as osp, mkdir
import matplotlib.pyplot as plt
import schedula as sh

log = logging.getLogger(__name__)
dsp = sh.BlueDispatcher(name='plot')


@sh.add_function(dsp)
def plot_and_save_simulation_result(starting_velocity, vehicle_id,
                                    gear_shifting_style, driver_style, times,
                                    velocities, accelerations, positions, start,
                                    stop, curves):
    """
    Plot simulation result.

    :param starting_velocity:
        Current velocity.
    :type starting_velocity: float (m/s)

    :param vehicle_id:
        Vehicle Id.
    :type vehicle_id: int

    :param gear_shifting_style:
        Gear shifting style.
    :type gear_shifting_style: float

    :param driver_style:
        Driver style.
    :type driver_style: float

    :param times:
        Sample time series.
    :type times: np.array

    :param velocities:
        Speed of the vehicle.
    :type velocities: list

    :param accelerations:
        Acceleration of vehicle.
    :type accelerations: list

    :param positions:
        Position of vehicle.
    :type positions: list

    :param start:
        Simulation start time.
    :type start: list

    :param stop:
        Simulation stop time.
    :type stop: list

    :param curves:
        Final acceleration curves.
    :type curves: list
    :return:
    """
    # Output folder to save simulation results.
    output_plot_dir = osp.dirname(__file__) + '/' + 'output'
    if not osp.exists(output_plot_dir):
        mkdir(output_plot_dir)

    # save velocity versus time plot
    fig1 = plt.figure('Velocity-Time')
    plt.plot(times, velocities[1:])
    plt.grid()
    fig1.savefig(output_plot_dir + '/' +
                 f"veh_{starting_velocity}_{vehicle_id}_{gear_shifting_style}_{driver_style}_time_velocity_plot.png", dpi=150)

    # save acceleration versus velocity plot
    fig2 = plt.figure('Acceleration-Velocity')
    plt.plot(velocities[1:], accelerations[1:])
    plt.grid()
    fig2.savefig(output_plot_dir + '/' +
                 f"veh_{starting_velocity}_{vehicle_id}_{gear_shifting_style}_{driver_style}_velocity_acceleration_plot.png", dpi=150)

    # save position versus time plot
    fig3 = plt.figure('Position-Time')
    plt.plot(times, positions[1:])
    plt.grid()
    fig3.savefig(output_plot_dir + '/' +
                 f"veh_{starting_velocity}_{vehicle_id}_{gear_shifting_style}_{driver_style}_acceleration_time_plot.png", dpi=150)

    # save acceleration versus time plot
    fig4 = plt.figure('Acceleration-Time')
    plt.plot(times, accelerations[1:])
    plt.grid()
    fig4.savefig(output_plot_dir + '/' +
                 f"veh_{starting_velocity}_{vehicle_id}_{gear_shifting_style}_{driver_style}_acceleration_time_plot.png", dpi=150)

    plt.figure('Acceleration-Velocity')
    for i, gear_curve in enumerate(curves):
        sp_bins = np.arange(start[i], stop[i] + 0.1, 0.1)
        accelerations = gear_curve(sp_bins)
        plt.plot(sp_bins, accelerations, 'k')
    plt.grid()
    # plt.show()

    return 0


if __name__ == '__main__':
    plot_and_save_simulation_result()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2015-2019 European Commission (JRC);
# Licensed under the EUPL (the 'Licence');
# You may not use this work except in compliance with the Licence.
# You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
import unittest
from co2mpas_driver.model import dsp
import ddt
from tests.utils import _check

import yaml
import os.path as osp


@ddt.ddt
class TestProcess(unittest.TestCase):
    def setUp(self):
        self.data, res_dir = {}, osp.join(osp.dirname(__file__), 'results')
        test_names = (
            'gear_4degree_curves_with_linear_gs', 'gear_curves_n_gs_from_poly',
            'get_ev_curve_main', 'sample_gear_shifting', 'sample_mfc_curves',
            'sample_EV'
        )
        for name in test_names:
            with open(osp.join(res_dir, '%s.yaml' % name)) as f:
                self.data[name] = yaml.load(f, yaml.CLoader)

    @ddt.idata((
            # `get_speeds_n_accelerations_per_gear`.
            ['speed_per_gear_updated', 'acc_per_gear_updated'],
            # `get_tan_coefs`.
            ['coefs_per_gear'],
            # `get_start_stop`.
            ['start', 'stop', 'sp_bins'],
            # `get_load_speed_n_torque`.
            ['full_load_speeds', 'full_load_torques'],
            # `get_resistances`.
            ['Alimit'],
            # `gear_linear`.
            ['gs'],
            # `get_spline_out_of_coefs`.
            ['discrete_poly_spline'],
            # `define_discrete_car_res_curve`
            ['discrete_car_res_curve'],
            # `define_discrete_car_res_curve_force`
            ['discrete_car_res_curve_force'],
            # `define_discrete_acceleration_curves`.
            ['discrete_acceleration_curves'],
    ))
    def test_gear_4degree_curves_with_linear_gs(self, out):
        _check(dsp, self.data['gear_4degree_curves_with_linear_gs'], out)

    @ddt.idata((
            # `calculate_full_load_speeds_and_powers`.
            ['full_load_powers', 'full_load_speeds'],
            # `calculate_full_load_torques`.
            ['full_load_torques'],
            # `get_speeds_n_accelerations_per_gear`.
            ['speed_per_gear', 'acc_per_gear'],
            # `get_tan_coefs`.
            ['coefs_per_gear'],
            # `get_start_stop`.
            ['start', 'stop'],
            ['gs'],
            # `define_discrete_poly`.
            ['discrete_poly_spline'],
            # `define_discrete_car_res_curve`.
            ['discrete_car_res_curve'],
            # `define_discrete_car_res_curve_force`.
            ['discrete_car_res_curve_force'],
            # `define_discrete_acceleration_curves`.
            ['discrete_acceleration_curves'],
            # `gear_points_from_tan`.
            # `get_resistances`.
            ['Alimit'],
    ))
    def test_gear_curves_n_gs_from_poly(self, out):
        _check(dsp, self.data['gear_curves_n_gs_from_poly'], out)

    @ddt.idata((
            # `define_discrete_acceleration_curves`.
            ['discrete_acceleration_curves'],
            # `get_resistances`.
            ['Alimit'],
            # `ev_curve`.
            ['start', 'stop'],
    ))
    def test_get_ev_curve_main(self, out):
        _check(dsp, self.data['get_ev_curve_main'], out)

    @ddt.idata((
            # `get_load_speed_n_torque`.
            ['full_load_speeds', 'full_load_torques'],
            # `get_speeds_n_accelerations_per_gear`.
            ['speed_per_gear', 'acc_per_gear'],
            # `get_tan_coefs`.
            ['coefs_per_gear'],
            # `get_start_stop`.
            ['start', 'stop', 'sp_bins'],
            # `get_cubic_splines_of_speed_acceleration_relationship`.
            ['discrete_poly_spline'],
            # `gear_points_from_tan`.
            ['gs'],
    ))
    def test_sample_gear_shifting(self, out):
        _check(dsp, self.data['sample_gear_shifting'], out)

    @ddt.idata((
            # `get_start_stop`.
            ['start', 'stop', 'sp_bins'],
            # `define_discrete_poly`.
            ['discrete_poly_spline'],
            # `define_discrete_car_res_curve`.
            ['discrete_car_res_curve'],
            # `define_discrete_car_res_curve_force`.
            ['discrete_car_res_curve_force'],
    ))
    def test_sample_ev(self, out):
        _check(dsp, self.data['sample_EV'], out)

    @ddt.idata((
            # `get_load_speed_n_torque`.
            ['full_load_speeds', 'full_load_torques'],
            # `get_speeds_n_accelerations_per_gear`.
            ['speed_per_gear', 'acc_per_gear'],
            # ``
            ['speed_per_gear_updated', 'acc_per_gear_updated'],
            # `get_tan_coefs`.
            ['coefs_per_gear'],
            # `get_start_stop`.
            ['start', 'stop', 'sp_bins'],
            # `define_discrete_poly`.
            ['discrete_poly_spline'],
            # `define_discrete_car_res_curve`.
            ['discrete_car_res_curve'],
            # `define_discrete_car_res_curve_force`.
            ['discrete_car_res_curve_force'],
            # `gear_linear`.
            ['gs'],
            # `define_discrete_acceleration_curves`.
            ['discrete_acceleration_curves'],
    ))
    def test_sample_mfc_curves(self, out):
        _check(dsp, self.data['sample_mfc_curves'], out)



from co2mpas_driver.common import vehicle_functions as vf
import numpy as np
from co2mpas_driver.common import gear_functions as fg


def gear_curves(my_car):
    """
    Calulate Full load curves of speed and torque

    :param my_car:
    :return:
    """

    full_load_speeds, full_load_torque = vf.get_load_speed_n_torque(my_car)

    '''Speed and acceleration ranges and poitns for each gear'''
    speed_per_gear, acc_per_gear = vf.get_speeds_n_accelerations_per_gear(my_car, full_load_speeds, full_load_torque)

    '''Extract speed acceleration Cubic Slpines'''
    cs_acc_per_gear = vf.get_cubic_splines_of_speed_acceleration_relationship(my_car, speed_per_gear, acc_per_gear)

    '''Start/stop speed for each gear'''
    Start, Stop = vf.get_start_stop(my_car, speed_per_gear, acc_per_gear, cs_acc_per_gear)

    sp_bins = np.arange(0, Stop[-1] + 1, 0.1)

    '''Get resistances'''
    car_res_curve, car_res_curve_force, Alimit = vf.get_resistances(my_car, sp_bins)

    '''Calculate Curves'''
    Curves = vf.calculate_curves_to_use(cs_acc_per_gear, Start, Stop, Alimit, car_res_curve, sp_bins)

    return Curves, cs_acc_per_gear, (Start, Stop)


def gear_curves_n_gs(my_car, gs_style, degree):
    """
    Calculate Full load curves of speed and torque
    Not used in the last version of MFC (for reference).

    :param my_car:
        Vehicle data base.

    :param gs_style:
        Gear shifting style.
    :type gs_style: float

    :param degree:
        Degree.
    :type degree: float

    :return:
    """
    full_load_speeds, full_load_torque = vf.get_load_speed_n_torque(my_car)

    '''Speed and acceleration ranges and poitns for each gear'''
    speed_per_gear, acc_per_gear = vf.get_speeds_n_accelerations_per_gear(my_car, full_load_speeds, full_load_torque)

    '''Extract speed acceleration Cubic Slpines'''
    cs_acc_per_gear = vf.get_cubic_splines_of_speed_acceleration_relationship(my_car, speed_per_gear, acc_per_gear)

    '''Start/stop speed for each gear'''
    Start, Stop = vf.get_start_stop(my_car, speed_per_gear, acc_per_gear, cs_acc_per_gear)

    sp_bins = np.arange(0, Stop[-1] + 1, 0.1)
    '''Get resistances'''
    car_res_curve, car_res_curve_force, Alimit = vf.get_resistances(my_car, sp_bins)

    '''Calculate Curves'''
    Curves = vf.calculate_curves_to_use(cs_acc_per_gear, Start, Stop, Alimit, car_res_curve, sp_bins)

    '''Get gs'''
    coefs_per_gear = vf.get_tan_coefs(speed_per_gear, acc_per_gear, degree)
    Tans = fg.find_list_of_tans_from_coefs(coefs_per_gear, Start, Stop)
    gs = fg.gear_points_from_tan(Tans, gs_style, Start, Stop)

    return Curves, cs_acc_per_gear, (Start, Stop), gs


def gear_curves_n_gs_from_poly(my_car, gs_style, degree):
    """
    Full load curves of speed and torque.

    :param my_car:
    :param gs_style:
    :param degree:
    :return:
    """

    full_load_speeds, full_load_torque = vf.get_load_speed_n_torque(my_car)

    '''Speed and acceleration ranges and poitns for each gear'''
    speed_per_gear, acc_per_gear = vf.get_speeds_n_accelerations_per_gear(my_car, full_load_speeds, full_load_torque)

    '''Extract speed acceleration Splines'''
    coefs_per_gear = vf.get_tan_coefs(speed_per_gear, acc_per_gear, degree)
    poly_spline = vf.get_spline_out_of_coefs(coefs_per_gear, speed_per_gear[0][0])

    '''Start/stop speed for each gear'''
    Start, Stop = vf.get_start_stop(my_car, speed_per_gear, acc_per_gear, poly_spline)

    sp_bins = np.arange(0, Stop[-1] + 1, 0.1)
    '''Get resistances'''
    car_res_curve, car_res_curve_force, Alimit = vf.get_resistances(my_car, sp_bins)

    '''Calculate Curves'''
    Curves = vf.calculate_curves_to_use(poly_spline, Start, Stop, Alimit, car_res_curve, sp_bins)

    Tans = fg.find_list_of_tans_from_coefs(coefs_per_gear, Start, Stop)
    gs = fg.gear_points_from_tan(Tans, gs_style, Start, Stop)

    return Curves, poly_spline, (Start, Stop), gs


def gear_4degree_curves_with_linear_gs(my_car, gs_style):
    """
    Full load curves of speed and torque.

    :param my_car:
    :param gs_style:
    :return:
    """
    full_load_speeds, full_load_torque = vf.get_load_speed_n_torque(my_car)

    '''Speed and acceleration ranges and poitns for each gear'''
    speed_per_gear, acc_per_gear = vf.get_speeds_n_accelerations_per_gear(my_car, full_load_speeds, full_load_torque)

    '''Extract speed acceleration Splines'''
    coefs_per_gear = vf.get_tan_coefs(speed_per_gear, acc_per_gear, 4)
    poly_spline = vf.get_spline_out_of_coefs(coefs_per_gear, speed_per_gear[0][0])

    '''Start/stop speed for each gear'''
    Start, Stop = vf.get_start_stop(my_car, speed_per_gear, acc_per_gear, poly_spline)

    sp_bins = np.arange(0, Stop[-1] + 0.1, 0.1)
    '''Get resistances'''
    car_res_curve, car_res_curve_force, Alimit = vf.get_resistances(my_car, sp_bins)

    '''Calculate Curves'''
    Curves = vf.calculate_curves_to_use(poly_spline, Start, Stop, Alimit, car_res_curve, sp_bins)

    '''Get gs'''
    gs = fg.gear_linear(speed_per_gear, gs_style)

    return Curves, poly_spline, (Start, Stop), gs


def get_ev_curve_main(my_car):
    """
    Full load curves of speed and torque.

    :param my_car:
    :return:
    """
    cs_acc_per_gear, Start, Stop = vf.ev_curve(my_car)

    # sp_bins = np.arange(0, Stop[-1] + 1, 0.01)
    sp_bins = np.arange(0, Stop[-1] + 0.1, 0.1)
    '''Get resistances'''
    car_res_curve, car_res_curve_force, Alimit = vf.get_resistances(my_car, sp_bins)

    '''Calculate Curves'''
    Curves = vf.calculate_curves_to_use(cs_acc_per_gear, Start, Stop, Alimit, car_res_curve, sp_bins)

    ppar = [0.0045, -0.1710, -1.8835]
    dec_curves = np.poly1d(ppar)
    final_dec = []
    Curves_dec = []
    for k in range(len(sp_bins)):
        final_dec.append(dec_curves(sp_bins[k]))
    from scipy.interpolate import CubicSpline, interp1d
    Curves_dec.append(interp1d(sp_bins, final_dec))

    # return Curves, (Start, Stop)
    gs = []
    return Curves, Curves_dec, (Start, Stop), gs


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2014-2017 European Commission (JRC);
# Licensed under the EUPL (the 'Licence');
# You may not use this work except in compliance with the Licence.
# You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl

"""
It provides constants for the CO2MPAS formulas.
"""

import numpy as np
from co2mpas_driver.common import utils as co2_utl


#: Container of node default values.
class Values(co2_utl.Constants):
    #: Does the engine have selective catalytic reduction technology?
    has_selective_catalytic_reduction = False

    #: Does the engine have lean burn technology?
    has_lean_burn = False

    #: Does the gear box have some additional technology to heat up faster?
    has_gear_box_thermal_management = False

    #: Does the vehicle has periodically regenerating systems? [-].
    has_periodically_regenerating_systems = False

    #: Possible percentages of active cylinders [-].
    active_cylinder_ratios = (1.0,)

    #: Does the engine feature variable valve actuation? [-].
    engine_has_variable_valve_actuation = False

    #: NEDC cycle time [s].
    max_time_NEDC = 1180.0

    #: WLTP cycle time [s].
    max_time_WLTP = 1800.0

    #: Maximum velocity to consider the vehicle stopped [km/h].
    stop_velocity = 1.0 + np.finfo(np.float32).eps

    #: Maximum acceleration to be at constant velocity [m/s2].
    plateau_acceleration = 0.1 + np.finfo(np.float32).eps

    #: Does the vehicle have start/stop system?
    has_start_stop = True

    #: Does the engine have cylinder deactivation technology?
    engine_has_cylinder_deactivation = False

    #: Minimum vehicle engine speed [RPM].
    min_engine_on_speed = 10.0

    #: Minimum time of engine on after a start [s].
    min_time_engine_on_after_start = 4.0

    #: Time window used to apply gear change filters [s].
    change_gear_window_width = 4.0

    #: Alternator start window width [s].
    alternator_start_window_width = 4.0

    #: Maximum clutch window width [s].
    max_clutch_window_width = 2.0

    #: Threshold vehicle velocity for gear correction due to full load curve
    #: [km/h].
    max_velocity_full_load_correction = 100.0

    #: Maximum allowed negative current for the alternator being considered off
    #: [A].
    alternator_off_threshold = -1.0

    #: Air density [kg/m3].
    air_density = 1.2

    #: Angle slope [rad].
    angle_slope = 0.0

    #: A different preconditioning cycle was used for WLTP and NEDC?
    correct_f0 = False

    #: Final drive ratio [-].
    final_drive_ratio = 1.0

    #: Final drive efficiency [-].
    final_drive_efficiency = 1.0

    #: Number of wheel drive [-].
    n_wheel_drive = 2

    #: Apply the eco-mode gear shifting?
    fuel_saving_at_strategy = True

    #: Is the vehicle hybrid?
    is_hybrid = False

    #: Cold and hot gear box reference temperatures [°C].
    gear_box_temperature_references = (40.0, 80.0)

    #: Constant torque loss due to engine auxiliaries [N*m].
    auxiliaries_torque_loss = 0.0

    #: Constant power loss due to engine auxiliaries [kW].
    auxiliaries_power_loss = 0.0

    #: If the engine is equipped with any kind of charging.
    engine_is_turbo = True

    #: Start-stop activation time threshold [s].
    start_stop_activation_time = 30

    #: Standard deviation of idle engine speed [RPM].
    idle_engine_speed_std = 100.0

    #: Is an hot cycle?
    is_cycle_hot = False

    #: CO2 emission model params.
    co2_params = {}

    #: Enable the calculation of Willans coefficients for all phases?
    enable_phases_willans = False

    #: Enable the calculation of Willans coefficients for the cycle?
    enable_willans = False

    #: Alternator efficiency [-].
    alternator_efficiency = 0.67

    #: Time elapsed to turn on the engine with electric starter [s].
    delta_time_engine_starter = 1.0

    #: If to use decision tree classifiers to predict gears.
    use_dt_gear_shifting = False

    #: Does the vehicle have energy recuperation features?
    has_energy_recuperation = True

    #: A/T Time at cold hot transition phase [s].
    time_cold_hot_transition = 300.0

    #: Time frequency [1/s].
    time_sample_frequency = 1.0

    #: Initial temperature of the test cell of NEDC [°C].
    initial_temperature_NEDC = 25.0

    #: Initial temperature of the test cell of WLTP [°C].
    initial_temperature_WLTP = 23.0

    #: K1 NEDC parameter (first or second gear) [-].
    k1 = 1

    #: K2 NEDC parameter (first or second gear) [-].
    k2 = 2

    #: K5 NEDC parameter (first or second gear) [-].
    k5 = 2

    #: WLTP base model params.
    wltp_base_model = {}

    #: Velocity downscale factor threshold [-].
    downscale_factor_threshold = 0.01

    #: Calibration torque converter speeds delta threshold [RPM].
    calibration_tc_speed_threshold = 100.0

    #: Limits (vel, acc) when torque converter is active [km/h, m/s].
    lock_up_tc_limits = (48.0, 0.3)

    #: Empirical value in case of CVT [-].
    tyre_dynamic_rolling_coefficient = 3.05 / 3.14


#: Container of internal function parameters.
class Functions(co2_utl.Constants):
    class select_prediction_data(co2_utl.Constants):
        #: If True the theoretical WLTP will be predicted, otherwise the driven.
        theoretical = True

    class calibrate_clutch_prediction_model(co2_utl.Constants):
        #: Delta time that identifies the previous engine speed to calibrate
        #: the clutch model [s].
        prev_dt = 1

    class _tech_mult_factors(co2_utl.Constants):
        #: Multiplication factors of the engine parameters [-].
        factors = {
            'vva': {
                #0: {},
                1: {'a': 0.98, 'l': 0.92},
            },
            'lb': {
                #0: {},
                1: {'a': 1.1, 'b' : 0.72, 'c': 0.76, 'a2': 1.25, 'l2': 2.85}
            },
            'egr': {
                #0: {},
                1: {'a': 1.02, 'b': 1.1, 'c': 1.5, 'a2': 1.1},  # positive turbo
                2: {'a': 1.02, 'b': 1.1, 'c': 1.5, 'a2': 1.1},  # positive natural aspiration
                3: {'b': 1.08, 'c': 1.15, 'a2': 1.1}, # compression
                4: {'b': 1.08, 'c': 1.15, 'a2': 1.1}  # compression + scr
            }
        }

    class FMEP_egr(co2_utl.Constants):
        #: Exhausted gas recirculation multiplication factors ids [-].
        egr_fact_map = {
            ('positive turbo', False): 1,
            ('positive natural aspiration', False): 2,
            ('compression', False): 3,
            ('compression', True): 4
        }

    class calibrate_co2_params(co2_utl.Constants):
        #: Enable third step in the co2_params calibration? [-]
        enable_third_step = True

        #: Use co2 error function against co2_emissions on the third step? [-]
        third_step_against_emissions = False


    class identify_co2_emissions(co2_utl.Constants):
        #: Number of perturbations to identify the co2_emissions [-].
        n_perturbations = 100

        #: Enable third step co2_params calibration in perturbation loop? [-]
        enable_third_step = True

        #: Use error function against co2_emissions in perturbation loop? [-]
        third_step_against_emissions = True

        #: Absolute error in k_refactor between iterations that is acceptable
        #: for convergence in perturbation loop [-].
        xatol = 1e-4

    class Alternator_status_model(co2_utl.Constants):
        #: Minimum delta time to consider valid a charging state to fit charges
        #: boundaries [s].
        min_delta_time_boundaries = 5

        #: Minimum acceptance percentile to fit the bers threshold [%].
        min_percentile_bers = 90

        #: Minimum delta soc to set the charging boundaries [%].
        min_delta_soc = 8

    class default_ki_factor(co2_utl.Constants):
        #: Correction for vehicles with periodically regenerating systems [-].
        ki_factor = {True: 1.05, False: 1.0}

    class define_fmep_model(co2_utl.Constants):
        #: Percentage of max full bmep curve used as limit in cylinder
        #: deactivation strategy [-].
        acr_full_bmep_curve_percentage = 0.45

        #: Percentage of max mean piston speeds used as limit in cylinder
        #: deactivation strategy [-].
        acr_max_mean_piston_speeds_percentage = 0.6

        #: Percentage of max full bmep curve used as limit in lean burn
        #: strategy [-].
        lb_full_bmep_curve_percentage = 0.4

        #: Percentage of max mean piston speeds used as limit in lean burn
        #: strategy [-].
        lb_max_mean_piston_speeds_percentage = 0.6

        #: Percentage of max full bmep curve used as limit in exhausted gas
        #: recirculation strategy [-].
        egr_full_bmep_curve_percentage = 0.5

        #: Percentage of max mean piston speeds used as limit in exhausted gas
        #: recirculation strategy [-].
        egr_max_mean_piston_speeds_percentage = 0.5

    class define_idle_model_detector(co2_utl.Constants):
        #: eps parameter of DBSCAN [RPM].
        EPS = 100.0

    class identify_idle_engine_speed_std(co2_utl.Constants):
        #: Min standard deviation value [RPM].
        MIN_STD = 100.0

        #: Max standard deviation percentage of median value [-].
        MAX_STD_PERC = 0.3

    class DefaultStartStopModel(co2_utl.Constants):
        #: Maximum allowed velocity to stop the engine [km/h].
        stop_velocity = 2.0

        #: Minimum acceleration to switch on the engine [m/s2].
        plateau_acceleration = 1 / 3.6

    class correct_constant_velocity(co2_utl.Constants):
        #: Constant velocities to correct the upper limits for NEDC [km/h].
        CON_VEL_UP_SHIFT = (15.0, 32.0, 50.0, 70.0)

        #: Window to identify if the shifting matrix has limits close to
        # `CON_VEL_UP_SHIFT` [km/h].
        VEL_UP_WINDOW = 3.5

        #: Delta to add to the limit if this is close to `CON_VEL_UP_SHIFT`
        # [km/h].
        DV_UP_SHIFT = -0.5

        #: Constant velocities to correct the bottom limits for NEDC[km/h].
        CON_VEL_DN_SHIFT = (35.0, 50.0)

        #: Window to identify if the shifting matrix has limits close to
        # `CON_VEL_DN_SHIFT` [km/h].
        VEL_DN_WINDOW = 3.5

        #: Delta to add to the limit if this is close to `CON_VEL_DN_SHIFT`
        # [km/h].
        DV_DN_SHIFT = -1

    class define_initial_co2_emission_model_params_guess(co2_utl.Constants):
        #: Initial guess CO2 emission model params.
        CO2_PARAMS = {
            'positive turbo': {
                'a': {'value': 0.468678},  # 'min': 0.398589, 'max': 0.538767},
                'b': {'value': 0.011859},  # 'min': 0.006558, 'max': 0.01716},
                'c': {'value': -0.00069},  # 'min': -0.00099, 'max': -0.00038},
                'a2': {'value': -0.00266},  # 'min': -0.00354, 'max': -0.00179},
                'b2': {'value': 0, 'min': -1, 'max': 1, 'vary': False},
                'l': {'value': -2.14063},  # 'min': -3.17876, 'max': -1.1025},
                'l2': {'value': -0.0025},  # 'min': -0.00796, 'max': 0.0},
                't0': {'value': 4.5},  # 'min': 0.0, 'max': 8.0},
                't1': {'value': 3.5},  # 'min': 0.0, 'max': 8.0},
            },
            'positive natural aspiration': {
                'a': {'value': 0.4851},  # 'min': 0.40065, 'max': 0.54315},
                'b': {'value': 0.01193},  # 'min': -0.00247, 'max': 0.026333},
                'c': {'value': -0.00065},  # 'min': -0.00138, 'max': 0.0000888},
                'a2': {'value': -0.00385},  # 'min': -0.00663, 'max': -0.00107},
                'b2': {'value': 0, 'min': -1, 'max': 1, 'vary': False},
                'l': {'value': -2.39882},  # 'min': -3.27698, 'max': -1.72066},
                'l2': {'value': -0.00286},  # 'min': -0.00577, 'max': 0.0},
                't0': {'value': 4.5},  # 'min': 0.0, 'max': 8.0},
                't1': {'value': 3.5},  # 'min': 0.0, 'max': 8.0},
            },
            'compression': {
                'a': {'value': 0.391197},  # 'min': 0.346548, 'max': 0.435846},
                'b': {'value': 0.028604},  # 'min': 0.002519, 'max': 0.054688},
                'c': {'value': -0.00196},  # 'min': -0.00386, 'max': -0.000057},
                'a2': {'value': -0.0012},  # 'min': -0.00233, 'max': -0.000064},
                'b2': {'value': 0, 'min': -1, 'max': 1, 'vary': False},
                'l': {'value': -1.55291},  # 'min': -2.2856, 'max': -0.82022},
                'l2': {'value': -0.0076},  # 'min': -0.01852, 'max': 0.0},
                't0': {'value': 4.5},  # 'min': 0.0, 'max': 8.0},
                't1': {'value': 3.5},  # 'min': 0.0, 'max': 8.0},
            }
        }

    class identify_charging_statuses(co2_utl.Constants):
        time_window = 4

    class restrict_bounds(co2_utl.Constants):
        #: Multipliers applied into the `restrict_bounds` function.
        CO2_PARAMS_LIMIT_MULTIPLIERS = {
            't0': (0.5, 1.5), 't1': (0.5, 1.5), 'trg': (0.9, 1.1),
            'a': (0.8, 1.2), 'b': (0.8, 1.2), 'c': (1.2, 0.8),
            'a2': (1.2, 0.8), 'l': (1.2, 0.8), 'l2': (1.2, 0.0),
        }

    class default_specific_gear_shifting(co2_utl.Constants):
        #: Specific gear shifting model.
        SPECIFIC_GEAR_SHIFTING = 'ALL'

    class default_clutch_k_factor_curve(co2_utl.Constants):
        #: Torque ratio when speed ratio==0 for clutch model.
        STAND_STILL_TORQUE_RATIO = 1.0

        #: Minimum speed ratio where torque ratio==1 for clutch model.
        LOCKUP_SPEED_RATIO = 0.0

    class default_tc_k_factor_curve(co2_utl.Constants):
        #: Torque ratio when speed ratio==0 for torque converter model.
        STAND_STILL_TORQUE_RATIO = 1.9

        #: Minimum speed ratio where torque ratio==1 for torque converter model.
        LOCKUP_SPEED_RATIO = 0.87

    class select_default_n_dyno_axes(co2_utl.Constants):
        #: Number of dyno axes [-].
        DYNO_AXES = {'WLTP': 2, 'NEDC': 1}

    class select_phases_integration_times(co2_utl.Constants):
        #: Cycle phases integration times [s].
        INTEGRATION_TIMES = {
            'WLTP': (0.0, 590.0, 1023.0, 1478.0, 1800.0),
            'NEDC': (0.0, 780.0, 1180.0)
        }

    class get_gear_box_efficiency_constants(co2_utl.Constants):
        #: Vehicle gear box efficiency constants (gbp00, gbp10, and gbp01).
        PARAMS = {
            True: {
                'gbp00': {'m': -0.0034, 'q': {'hot': -0.3, 'cold': -0.7}},
                'gbp10': {'m': -0.0034 / 2000, 'q': {'hot': -0.3 / 2000,
                                                     'cold': -1 / 2000}},
                'gbp01': {'q': {'hot': 0.965, 'cold': 0.955}},
            },
            False: {
                'gbp00': {'m': -0.0034, 'q': {'hot': -0.3, 'cold': -0.7}},
                'gbp10': {'m': -0.0034 / 2000, 'q': {'hot': -0.1 / 2000,
                                                     'cold': -0.25 / 2000}},
                'gbp01': {'q': {'hot': 0.975, 'cold': 0.965}},
            }
        }

    class calculate_engine_mass(co2_utl.Constants):
        #: Equivalent gear box heat capacity parameters.
        PARAMS = {
            'mass_coeff': {
                'compression': 1.1,
                'positive': 1.0
            },
            'mass_reg_coeff': (0.4208, 60)
        }

    class calculate_engine_heat_capacity(co2_utl.Constants):
        #: Equivalent gear box heat capacity parameters.
        PARAMS = {
            'heated_mass_percentage': {
                'coolant': 0.04,     # coolant: 50%/50% (0.85*4.186)
                'oil': 0.055,        # oil: lubricant
                'crankcase': 0.18,   # crankcase: cast iron
                'cyl_head': 0.09,    # cyl_head: aluminium
                'pistons': 0.025,    # pistons: aluminium
                'crankshaft': 0.08,  # crankshaft: steel
                'body': 0.1          # body: cast iron
            },
            # Cp in (J/kgK)
            'heat_capacity': {
                'coolant': 0.85 * 4186.0,
                'oil': 2090.0,
                'crankcase': 460.0,
                'cyl_head': 910.0,
                'pistons': 910.0,
                'crankshaft': 490.0,
                'body': 460.0
            }
        }

    class calculate_equivalent_gear_box_heat_capacity(co2_utl.Constants):
        #: Equivalent gear box heat capacity parameters.
        PARAMS = {
            'gear_box_mass_engine_ratio': 0.25,
            # Cp in (J/kgK)
            'heat_capacity': {
                'oil': 2090.0,
            },
            'thermal_management_factor': 0.5
        }

    class get_full_load(co2_utl.Constants):
        #: Vehicle normalized full load curve.
        FULL_LOAD = {
            'positive': (
                [0., 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1., 1.1, 1.2],
                [0.1, 0.198238659, 0.30313392, 0.410104642, 0.516920841,
                 0.621300767, 0.723313491, 0.820780368, 0.901750158,
                 0.962968496, 0.995867804, 0.953356174, 0.85]
            ),
            'compression': (
                [0., 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1., 1.1, 1.2],
                [0.1, 0.278071182, 0.427366185, 0.572340499, 0.683251935,
                 0.772776746, 0.846217049, 0.906754984, 0.94977083, 0.981937981,
                 1, 0.937598144, 0.85]
            )
        }

    class calculate_engine_max_torque(co2_utl.Constants):
        #: Engine nominal torque params.
        PARAMS = {
            'positive': 1.25,
            'compression': 1.1
        }

    class calculate_engine_moment_inertia(co2_utl.Constants):
        #: Engine moment of inertia params.
        PARAMS = {
            'positive': 1,
            'compression': 2
        }

    class calculate_co2_emissions(co2_utl.Constants):
        # idle ratio to define the fuel cutoff [-].
        cutoff_idle_ratio = 1.1

    class default_initial_state_of_charge(co2_utl.Constants):
        # default initial state of charge of the battery [%].
        initial_state_of_charge = {
            'WLTP': 90,
            'NEDC': 99
        }

    # TODO: add default fuel densities.
    class default_fuel_density(co2_utl.Constants):
        #: Fuel density [g/l].
        FUEL_DENSITY = {
            'gasoline': 750.0,
            'diesel': 835.0,
            # 'LPG': ,
            # 'NG': ,
            # 'ethanol': ,
            # 'biodiesel': ,
        }

    # TODO: add default fuel lower heating values.
    class default_fuel_lower_heating_value(co2_utl.Constants):
        #: Fuel lower heating value [kJ/kg].
        LHV = {
            # 'gasoline': ,
            # 'diesel': ,
            # 'LPG': ,
            # 'NG': ,
            # 'ethanol': ,
            # 'biodiesel': ,
        }

    # TODO: add default fuel carbon content.
    class default_fuel_carbon_content(co2_utl.Constants):
        #: Fuel carbon content [CO2g/g].
        CARBON_CONTENT = {
            # 'gasoline': ,
            # 'diesel': ,
            # 'LPG': ,
            # 'NG': ,
            # 'ethanol': ,
            # 'biodiesel': ,
        }

    class calibrate_cold_start_speed_model_v1(co2_utl.Constants):
        #: Cold start engine speed model v1 params.
        PARAMS = {
            'first_seconds': 10.0,  # [s]
            'delta_speed_limits': (0.05, 0.2),  # [-, -]
            'max_temperature': 30.0  # [°C]
        }

    class calculate_cold_start_speeds_delta(co2_utl.Constants):
        #: Maximum cold start speed delta percentage of idle [-].
        MAX_COLD_START_SPEED_DELTA_PERCENTAGE = 1.0

    class _yield_on_start(co2_utl.Constants):
        #: Minimum velocity that allow to switch off stop the engine after an
        #: off [km/h].
        VEL = 1.0

        #: Minimum acceleration that allow to switch off stop the engine after
        #: an off [m/s2].
        ACC = 0.05


class Defaults(co2_utl.Constants):
    values = Values()
    functions = Functions()

    #: Machine error.
    EPS = np.finfo(np.float32).eps

    #: Infinite value.
    INF = 10000.0

dfl = Defaults()



from co2mpas_driver.common import gear_functions as fg


def accMFC(s, driver_style, sdes, acc_p_curve):
    '''

    Return the MFC free flow acceleration

    :param s:                   speed (m/s)
    :param driver_style:        ds 0-1
    :param sdes:                desired speed (m/s)
    :param acc_p_curve:         speed acceleration curve of the gear in use
    :return:
    '''
    if s / sdes > 0.5:
        if sdes > s:
            onoff = (1 - pow(s / sdes, 60))
        else:
            onoff = 10 * (1 - s / sdes)
    else:
        onoff = (1 - 0.8 * pow(1 - s / sdes , 60))
    acc = acc_p_curve(s) * driver_style * onoff

    return acc


def clutch_on(gear_count, acc, my_car):
    '''

    If clutch is on, maximum acceleration is decreased depending on the transmission

    :param gear_count:
    :param acc:
    :param my_car:
    :return:
    '''

    if gear_count > 0:

        if my_car.transmission == 'manual':
            return 0.
        else:
            return acc * 2 / 3
    else:
        return acc


def simulation_step_function(selected_car, speed, gear, gear_count, gs, Curves, vdes, driver_style, sim_step):
    gear, gear_count = fg.gear_for_speed_profiles(gs, speed, gear, gear_count)
    acceleration = accMFC(speed, driver_style, vdes, Curves[gear - 1])
    acceleration = clutch_on(gear_count, acceleration, selected_car)

    return speed + acceleration * sim_step, acceleration, gear, gear_count



# -*- coding: utf-8 -*-
#
# Copyright 2019 European Commission (JRC);
# Licensed under the EUPL (the 'Licence');
# You may not use this work except in compliance with the Licence.
# You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
"""
Functions and a model `dsp` to processes a CO2MPAS input file.

Sub-Modules:
.. currentmodule:: co2mpas_driver
.. autosummary::
    :nosignatures:
    :toctree: _build/co2mpas_driver/
    load
    model
    write
    plot
"""
import matplotlib.pyplot as plt
import schedula as sh
from co2mpas_driver.load import dsp as _load
from co2mpas_driver.model import dsp as _model
from co2mpas_driver.plot import dsp as _plot

dsp = sh.Dispatcher()
dsp.add_dispatcher(
    dsp=_load,
    inputs=['inputs', 'vehicle_id', 'db_path', 'input_path'],
    outputs=['data']
)

dsp.add_function(
    function=sh.SubDispatch(_model),
    inputs=['data'],
    outputs=['outputs']
)

dsp.add_function(
    function_id='write',
    inputs=['output_path', 'outputs']
)

dsp.add_function(
    function=sh.SubDispatch(_plot),
    inputs=['output_plot_folder', 'outputs']
)

if __name__ == '__main__':
    sol = dsp(dict(vehicle_id=39393, inputs=dict(inputs=dict(
        gear_shifting_style=0.9, desired_velocity=124/3.6,
        starting_velocity=0, degree=4, driver_style=1,
        sim_start=0, sim_step=0.1, duration=100, use_linear_gs=True,
        use_cubic=False))))[
        'outputs']
    # velocity = sol['outputs']['velocities']
    # acceleration = sol['outputs']['accelerations']
    # plt.figure('Speed-Acceleration')
    # plt.plot(velocity[1:], acceleration[1:])
    # plt.grid()
    # plt.show()
    dsp.plot()